@@ -133,6 +133,7 @@ static const char* cone_name_long = "Cone<long long>";
 static const char* cone_name_renf = "Cone<renf_elem>";
 
 static PyObject* RationalHandler = NULL;
+static PyObject* FloatHandler = NULL;
 
 #ifdef ENFNORMALIZ
 static PyObject* NumberfieldElementHandler = NULL;
@@ -171,11 +172,11 @@ static PyObject* CallPythonFuncOnOneArg(PyObject* function, PyObject* single_arg
 #ifndef NMZ_RELEASE
 static_assert(
     false,
-    "Your Normaliz version (unknown) is to old! Update to 3.8.5 or newer.");
+    "Your Normaliz version (unknown) is to old! Update to 3.8.9 or newer.");
 #endif
-#if NMZ_RELEASE < 30805
+#if NMZ_RELEASE < 30809
 static_assert(false,
-              "Your Normaliz version is to old! Update to 3.8.5 or newer.");
+              "Your Normaliz version is to old! Update to 3.8.9 or newer.");
 #endif
 
 /***************************************************************************
@@ -228,7 +229,7 @@ static bool PyNumberToNmz(PyObject*, mpz_class&);
 static bool PyNumberToNmz(PyObject* in, mpq_class& out)
 {
     if (PyFloat_Check(in)) {
-        out = mpq_class(PyFloat_AsDouble(in));
+        throw PyNormalizInputException("PyFloat not allowed in PyNormaliz input. Must be ecoded as string.");
         return true;
     }
 #if PY_MAJOR_VERSION < 3
@@ -262,12 +263,9 @@ static bool PyNumberToNmz(PyObject* in, mpq_class& out)
     }
     PyObject*   in_as_string = PyObject_Str(in);
     string      s = PyUnicodeToString(in_as_string);
-    int         check = out.set_str(s.c_str(), 10);
-    if (check == -1) {
-        throw PyNormalizInputException(
-            "coefficient in matrix must be PyFloat, PyInt, PyLong, Sequence, "
-            "must be able to be converted to a valid gmp input string");
-    }
+    // int         check = out.set_str(s.c_str(), 10);
+    libnormaliz::string2coeff(out,s);
+
     return true;
 }
 
@@ -355,7 +353,11 @@ static PyObject* NmzToPyNumber(long long in)
 
 static PyObject* NmzToPyNumber(double in)
 {
-    return PyFloat_FromDouble(in);
+    PyObject* x = PyFloat_FromDouble(in);
+    if(FloatHandler == NULL)
+        return x;
+ 
+    return CallPythonFuncOnOneArg(FloatHandler, x);
 }
 
 template < typename Integer >
@@ -365,23 +367,38 @@ static PyObject* NmzVectorToPyList(const vector< Integer >& in,
 #ifdef ENFNORMALIZ
 static PyObject* NmzToPyNumber(const renf_elem_class &in)
 {
+    // std::cout << "IIIII " << in << std::endl;
     vector< mpz_class > output_nums = in.get_num_vector();
     mpz_class           output_den = in.get_den();
-    PyObject*           denom_py = NmzToPyNumber(output_den);
+    vector< mpz_class > denoms(output_nums.size(), output_den);
+    for(size_t i=0; i< output_nums.size(); ++i){
+        mpq_class quot = output_nums[i];
+        quot /= output_den;
+        output_nums[i] = quot.get_num();
+        denoms[i] = quot.get_den();
+    }
+    /*std::cout << "NNN ";
+    for( size_t i = 0; i< output_nums.size(); ++i)
+        std::cout << output_nums[i] << " ";
+    std::cout << std::endl;
+    std::cout << "DDD ";
+    for( size_t i = 0; i< output_nums.size(); ++i)
+        std::cout << denoms[i] << " ";
+    std::cout << std::endl;*/
+    // PyObject*           denom_py = NmzToPyNumber(output_den);
     PyObject*           out_list = PyList_New(output_nums.size());
     for (size_t i = 0; i < output_nums.size(); i++) {
         PyObject* current = PyList_New(2);
         PyList_SetItem(current, 0, NmzToPyNumber(output_nums[i]));
-        Py_IncRef(denom_py);
-        PyList_SetItem(current, 1, denom_py);
+        // Py_IncRef(denom_py);
+        PyList_SetItem(current, 1, NmzToPyNumber(denoms[i]));
         if (RationalHandler != NULL)
             current = CallPythonFuncOnOneArg(RationalHandler, current);
         PyList_SetItem(out_list, i, current);
     }
-    Py_DecRef(denom_py);
+    // Py_DecRef(denom_py);
     if (NumberfieldElementHandler != NULL)
-        out_list =
-            CallPythonFuncOnOneArg(NumberfieldElementHandler, out_list);
+        out_list = CallPythonFuncOnOneArg(NumberfieldElementHandler, out_list);
     return out_list;
 }
 #endif
@@ -444,7 +461,8 @@ static bool prepare_nf_input(vector< vector< NumberFieldElem > >& out,
             PyObject* current_element = PySequence_GetItem(current_row, j);
             bool      current_res;
             NumberFieldElem current_elem;
-            if (PySequence_Check(current_element)) {
+    
+            if (PyList_CheckExact(current_element) || PyTuple_CheckExact(current_element)) {
                 vector< mpq_class > current_vector;
                 current_res = PyListToNmz(current_vector, current_element);
                 if (!current_res) {
@@ -1620,9 +1638,15 @@ _NmzResultImpl(Cone< Integer >* C, PyObject* prop_obj, void* nf = nullptr)
 
         case libnormaliz::ConeProperty::FVector:
             return NmzVectorToPyList(C->getFVector());
+            
+        case libnormaliz::ConeProperty::DualFVector:
+            return NmzVectorToPyList(C->getDualFVector());
 
         case libnormaliz::ConeProperty::FaceLattice:
             return NmzFacelatticeToPython(C->getFaceLattice());
+            
+        case libnormaliz::ConeProperty::DualFaceLattice:
+            return NmzFacelatticeToPython(C->getDualFaceLattice());
 
         case libnormaliz::ConeProperty::Automorphisms:
             return NmzAutomorphismsToPython(C->getAutomorphismGroup(
@@ -1689,7 +1713,17 @@ _NmzResultImpl(Cone< Integer >* C, PyObject* prop_obj, void* nf = nullptr)
 static PyObject* _NmzResult(PyObject* self, PyObject* args, PyObject* kwargs)
 {
 
-    FUNC_BEGIN
+FUNC_BEGIN
+    
+RationalHandler = NULL;
+FloatHandler = NULL;
+
+#ifdef ENFNORMALIZ
+NumberfieldElementHandler = NULL;
+#endif
+
+VectorHandler = NULL;
+MatrixHandler = NULL;
     
     if(PyTuple_Size(args)!=2){
         PyErr_SetString(PyNormaliz_cppError, "Exactly one computation goal required for NmzResult");
@@ -1712,6 +1746,7 @@ static PyObject* _NmzResult(PyObject* self, PyObject* args, PyObject* kwargs)
 
     if (kwargs) {
         RationalHandler = PyDict_GetItemString(kwargs, "RationalHandler");
+        FloatHandler = PyDict_GetItemString(kwargs, "FloatHandler");
 #ifdef ENFNORMALIZ
         NumberfieldElementHandler =
             PyDict_GetItemString(kwargs, "NumberfieldElementHandler");
